'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var jsxRuntime = require('react/jsx-runtime');
var react = require('react');

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var hexToRGB = function (hex, alpha) {
    if (alpha === void 0) { alpha = 1; }
    var parseString = hex;
    if (hex.startsWith('#')) {
        parseString = hex.slice(1, 7);
    }
    if (parseString.length !== 6) {
        return "rgba(255, 255, 255, ".concat(alpha, ")");
    }
    var r = parseInt(parseString.slice(0, 2), 16);
    var g = parseInt(parseString.slice(2, 4), 16);
    var b = parseInt(parseString.slice(4, 6), 16);
    if (isNaN(r) || isNaN(g) || isNaN(b)) {
        return "rgba(255, 255, 255, ".concat(alpha, ")");
    }
    return "rgba(".concat(r, ", ").concat(g, ", ").concat(b, ", ").concat(alpha, ")");
};

var IsDevice = (function () {
    if (typeof navigator == 'undefined')
        return;
    var ua = navigator.userAgent;
    return {
        info: ua,
        Android: function () {
            return ua.match(/Android/i);
        },
        BlackBerry: function () {
            return ua.match(/BlackBerry/i);
        },
        IEMobile: function () {
            return ua.match(/IEMobile/i);
        },
        iOS: function () {
            return ua.match(/iPhone|iPad|iPod/i);
        },
        iPad: function () {
            return (ua.match(/Mac/) &&
                navigator.maxTouchPoints &&
                navigator.maxTouchPoints > 2);
        },
        OperaMini: function () {
            return ua.match(/Opera Mini/i);
        },
        /**
         * Any Device
         */
        any: function () {
            return (IsDevice.Android() ||
                IsDevice.BlackBerry() ||
                IsDevice.iOS() ||
                IsDevice.iPad() ||
                IsDevice.OperaMini() ||
                IsDevice.IEMobile());
        }
    };
})();

/* eslint-disable react-hooks/exhaustive-deps */
/**
 * useEventListener
 * Hook for handling EventListeners
 * @return {object} width, height
 */
var useEventListener = function (eventName, 
// eslint-disable-next-line no-unused-vars
handler, element) {
    if (element === void 0) { element = window; }
    // Create a ref that stores handler
    var savedHandler = react.useRef(null);
    // Update ref.current value if handler changes.
    react.useEffect(function () {
        savedHandler.current = handler;
    }, [handler]);
    react.useEffect(function () {
        // Make sure element supports addEventListener
        var isSupported = element && element.addEventListener;
        if (!isSupported)
            return;
        // Create event listener that calls handler function stored in ref
        var eventListener = function (event) { return savedHandler.current(event); };
        // Add event listener
        element.addEventListener(eventName, eventListener);
        // Remove event listener on cleanup
        return function () {
            element.removeEventListener(eventName, eventListener);
        };
    }, [eventName, element] // Re-run if eventName or element changes
    );
};

/**
 * Cursor Core
 * Replaces the native cursor with a custom animated cursor, consisting
 * of an inner and outer dot that scale inversely based on hover or click.
 *
 * @author Stephen Scaff (github.com/stephenscaff)
 *
 * @param {string} color - rgb color value (RGB or HEX)
 * @param {number} outerAlpha - level of alpha transparency for color
 * @param {number} innerSize - inner cursor size in px
 * @param {number} innerScale - inner cursor scale amount
 * @param {number} outerSize - outer cursor size in px
 * @param {number} outerScale - outer cursor scale amount
 * @param {object} outerStyle - style object for outer cursor
 * @param {object} innerStyle - style object for inner cursor
 * @param {array}  clickables - array of clickable selectors
 *
 */
var CursorCore = function (_a) {
    var outerStyle = _a.outerStyle, innerStyle = _a.innerStyle, color = _a.color, outerAlpha = _a.outerAlpha, innerSize = _a.innerSize, innerScale = _a.innerScale, outerSize = _a.outerSize, outerScale = _a.outerScale, trailingSpeed = _a.trailingSpeed, clickables = _a.clickables;
    var cursorOuterRef = react.useRef(null);
    var cursorInnerRef = react.useRef(null);
    var requestRef = react.useRef(null);
    var previousTimeRef = react.useRef(null);
    var _b = react.useState({ x: 0, y: 0 }), coords = _b[0], setCoords = _b[1];
    var _c = react.useState(false), isVisible = _c[0], setIsVisible = _c[1];
    var _d = react.useState(false), isActive = _d[0], setIsActive = _d[1];
    var _e = react.useState(false), isActiveClickable = _e[0], setIsActiveClickable = _e[1];
    var endX = react.useRef(0);
    var endY = react.useRef(0);
    /**
     * Primary Mouse move event
     * @param {number} clientX - MouseEvent.clientx
     * @param {number} clientY - MouseEvent.clienty
     */
    var onMouseMove = react.useCallback(function (_a) {
        var clientX = _a.clientX, clientY = _a.clientY;
        setCoords({ x: clientX, y: clientY });
        cursorInnerRef.current.style.top = "".concat(clientY, "px");
        cursorInnerRef.current.style.left = "".concat(clientX, "px");
        endX.current = clientX;
        endY.current = clientY;
    }, []);
    // Outer Cursor Animation Delay
    var animateOuterCursor = react.useCallback(function (time) {
        if (previousTimeRef.current !== undefined) {
            coords.x += (endX.current - coords.x) / trailingSpeed;
            coords.y += (endY.current - coords.y) / trailingSpeed;
            cursorOuterRef.current.style.top = "".concat(coords.y, "px");
            cursorOuterRef.current.style.left = "".concat(coords.x, "px");
        }
        previousTimeRef.current = time;
        requestRef.current = requestAnimationFrame(animateOuterCursor);
    }, [requestRef] // eslint-disable-line
    );
    // RAF for animateOuterCursor
    react.useEffect(function () {
        requestRef.current = requestAnimationFrame(animateOuterCursor);
        return function () { return cancelAnimationFrame(requestRef.current); };
    }, [animateOuterCursor]);
    // Mouse Events State updates
    var onMouseDown = react.useCallback(function () { return setIsActive(true); }, []);
    var onMouseUp = react.useCallback(function () { return setIsActive(false); }, []);
    var onMouseEnterViewport = react.useCallback(function () { return setIsVisible(true); }, []);
    var onMouseLeaveViewport = react.useCallback(function () { return setIsVisible(false); }, []);
    useEventListener('mousemove', onMouseMove);
    useEventListener('mousedown', onMouseDown);
    useEventListener('mouseup', onMouseUp);
    useEventListener('mouseover', onMouseEnterViewport);
    useEventListener('mouseout', onMouseLeaveViewport);
    // Cursors Hover/Active State
    react.useEffect(function () {
        if (isActive) {
            cursorInnerRef.current.style.transform = "translate(-50%, -50%) scale(".concat(innerScale, ")");
            cursorOuterRef.current.style.transform = "translate(-50%, -50%) scale(".concat(outerScale, ")");
        }
        else {
            cursorInnerRef.current.style.transform = 'translate(-50%, -50%) scale(1)';
            cursorOuterRef.current.style.transform = 'translate(-50%, -50%) scale(1)';
        }
    }, [innerScale, outerScale, isActive]);
    // Cursors Click States
    react.useEffect(function () {
        if (isActiveClickable) {
            cursorInnerRef.current.style.transform = "translate(-50%, -50%) scale(".concat(innerScale * 1.2, ")");
            cursorOuterRef.current.style.transform = "translate(-50%, -50%) scale(".concat(outerScale * 1.4, ")");
        }
    }, [innerScale, outerScale, isActiveClickable]);
    // Cursor Visibility State
    react.useEffect(function () {
        if (isVisible) {
            cursorInnerRef.current.style.opacity = '1';
            cursorOuterRef.current.style.opacity = '1';
        }
        else {
            cursorInnerRef.current.style.opacity = '0';
            cursorOuterRef.current.style.opacity = '0';
        }
    }, [isVisible]);
    react.useEffect(function () {
        var clickableEls = document.querySelectorAll(clickables.join(','));
        clickableEls.forEach(function (el) {
            el.style.cursor = 'none';
            el.addEventListener('mouseover', function () {
                setIsActive(true);
            });
            el.addEventListener('click', function () {
                setIsActive(true);
                setIsActiveClickable(false);
            });
            el.addEventListener('mousedown', function () {
                setIsActiveClickable(true);
            });
            el.addEventListener('mouseup', function () {
                setIsActive(true);
            });
            el.addEventListener('mouseout', function () {
                setIsActive(false);
                setIsActiveClickable(false);
            });
        });
        return function () {
            clickableEls.forEach(function (el) {
                el.removeEventListener('mouseover', function () {
                    setIsActive(true);
                });
                el.removeEventListener('click', function () {
                    setIsActive(true);
                    setIsActiveClickable(false);
                });
                el.removeEventListener('mousedown', function () {
                    setIsActiveClickable(true);
                });
                el.removeEventListener('mouseup', function () {
                    setIsActive(true);
                });
                el.removeEventListener('mouseout', function () {
                    setIsActive(false);
                    setIsActiveClickable(false);
                });
            });
        };
    }, [isActive, clickables]);
    var mainStyles = {
        zIndex: 999,
        display: 'block',
        position: 'fixed',
        borderRadius: '50%',
        pointerEvents: 'none',
        transition: 'opacity 0.15s ease-in-out, transform 0.25s ease-in-out'
    };
    // Cursor Styles
    var styles = {
        cursorInner: __assign(__assign(__assign({}, mainStyles), { width: innerSize, height: innerSize, backgroundColor: color.includes('#')
                ? hexToRGB(color)
                : "rgba(".concat(color, ", 1)") }), (innerStyle && innerStyle)),
        cursorOuter: __assign(__assign(__assign({}, mainStyles), { width: outerSize, height: outerSize, backgroundColor: color.includes('#')
                ? hexToRGB(color, outerAlpha)
                : "rgba(".concat(color, ", ").concat(outerAlpha, " )"), willChange: 'transform' }), (outerStyle && outerStyle))
    };
    // Hide / Show global cursor
    document.body.style.cursor = 'none';
    return (jsxRuntime.jsxs(jsxRuntime.Fragment, { children: [jsxRuntime.jsx("div", { ref: cursorOuterRef, style: styles.cursorOuter }), jsxRuntime.jsx("div", { ref: cursorInnerRef, style: styles.cursorInner })] }));
};
/**
 * AnimatedCursor
 * Calls and passes props to CursorCore if not a touch/mobile device.
 */
var AnimatedCursor = function (_a) {
    var outerStyle = _a.outerStyle, innerStyle = _a.innerStyle, _b = _a.color, color = _b === void 0 ? '220, 90, 90' : _b, _c = _a.outerAlpha, outerAlpha = _c === void 0 ? 0.3 : _c, _d = _a.innerSize, innerSize = _d === void 0 ? 8 : _d, _e = _a.outerSize, outerSize = _e === void 0 ? 8 : _e, _f = _a.outerScale, outerScale = _f === void 0 ? 6 : _f, _g = _a.innerScale, innerScale = _g === void 0 ? 0.6 : _g, _h = _a.trailingSpeed, trailingSpeed = _h === void 0 ? 8 : _h, _j = _a.clickables, clickables = _j === void 0 ? [
        'a',
        'input[type="text"]',
        'input[type="email"]',
        'input[type="number"]',
        'input[type="submit"]',
        'input[type="image"]',
        'label[for]',
        'select',
        'textarea',
        'button',
        '.link'
    ] : _j;
    if (typeof navigator !== 'undefined' && IsDevice.any()) {
        return jsxRuntime.jsx(jsxRuntime.Fragment, {});
    }
    return (jsxRuntime.jsx(CursorCore, { outerStyle: outerStyle, innerStyle: innerStyle, color: color, outerAlpha: outerAlpha, innerSize: innerSize, innerScale: innerScale, outerSize: outerSize, outerScale: outerScale, trailingSpeed: trailingSpeed, clickables: clickables }));
};

exports.AnimatedCursor = AnimatedCursor;
